; ---USAGE---

; python3 ok <name_of_function>

; e.g. python3 ok is-prefix

; ---NOTES---

; - if you want to see the verbose output, run this:

;     python3 ok <name_of_function> -v

; - if you want to test all of your functions, run this:

;     python3 ok

;

;;; Discussion 12 - More Scheme and Interpreters ;;;

; Q3.1 - is-prefix
;;; Tests
; (is-prefix '() '())
; expect #t
; (is-prefix '() '(1 2))
; expect #t
; (is-prefix '(1) '(1 2))
; expect #t
; (is-prefix '(2) '(1 2))
; expect #f
; (is-prefix '(1 2) '(1)) ; Note here p is longer than lst
; expect #f

(define (is-prefix p lst)
  (if (null? p)
    #t
    (if (null? lst)
      #f
      (and
        (= (car p) (car lst))
        (is-prefix (cdr p) (cdr lst))
      )
    )
  )
)

; Alternate solution (using `cond`):

; (define (is-prefix p lst)
;   (cond
;     ((null? p) #t)
;     ((null? lst) #f)
;     (else (and (= (car p) (car lst))
;       (is-prefix (cdr p) (cdr lst)))
;     )
;   )
; )

; Q3.2 - apply-multiple
;;; Tests
; (apply-multiple (lambda (x) (* x x)) 3 2)
; expect 256
; (apply-multiple (lambda (x) (+ x 1)) 10 1)
; expect 11
; (apply-multiple (lambda (x) (* 1000 x)) 0 5)
; expect 5

(define (apply-multiple f n x)
  (if (= n 0)
    x
    (f (apply-multiple f (- n 1) x))
  )
)

; Alternate solution:

; (define (apply-multiple f n x)
;   (if (= n 0)
;     x
;     (apply-multiple f (- n 1) (f x))
;   )
; )

; Q3.3.1 - max
;;; Tests
; (max 1 5)
; expect 5

(define (max x y) (if (> x y) x y))

; Q3.3.2 - max-depth
;;; Tests
; (max-depth '(1 2 3))
; expect 0
; (max-depth '(1 2 (3 (4) 5)))
; expect 2
; (max-depth '(0 (1 (2 (3 (4) 5) 6) 7)))
; expect 4

(define (max-depth lst)
  (define (helper lst curr)
    (cond
      ((null? lst) curr)
      ((pair? (car lst)) (max (helper (car lst) (+ 1 curr))
                            (helper (cdr lst) curr)))
      (else (helper (cdr lst) curr))
    )
  )
  (helper lst 0)
)